package parMSpindle;
/*
 * Bug.java... a rod shaped bacteria with cylindrical midsection and hemispherical caps
 *
 */

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;

import javax.swing.*;
import java.lang.Math.*;
import java.text.DecimalFormat;

import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Bug extends Thing {
	static double length;			// length from end to end
	static double radius;			// radius of spherical caps and cylindrical section
	static double cellThickness = 0.00;		// thickness of the bacterial cell wall
	static double cylLength;
	static double halfCylLength;
	static double cylVolume;		// volume of the cylinder
	static double sphVolume;		// volume of the 2 hemispheres
	static double bugVolume;		// entire volume contained within the rod-shaped bug
	static double microMolarChangePerMonomer,milliMolarChangePerMonomer;	// how much to change concentration when one monomer is used or returned
	static Color bugColor = Color.white;
	static float initialTrans = 0.6f;	// initial transparency
	static float initialEmit = 0.1f;	// initial emmissive color
	
	static DecimalFormat timeFormat = new DecimalFormat ("0000.0000");
	static DecimalFormat concFormat = new DecimalFormat ("0.0000");
	
	// for graphics
	static BranchGroup bugG,infoG;
	static Cylinder myCyl1,myCyl2;
	static SphereSection lArch,rArch,lArchBony,rArchBony;
	static Color3f ambientC = new Color3f(0.0f,0.0f,0.0f);
	static Color3f diffuseC = new Color3f(0.9f,0.9f,0.9f);
	static Color3f specularC = new Color3f(1.0f,1.0f,1.0f);
	static Color3f emissiveC = new Color3f(0.1f,0.1f,0.1f);
	static float shiny = 60.0f;
	static Appearance aWire = new Appearance();
	static Appearance aSphCaps, bonyApp;
	static TransparencyAttributes noSeeMeTA, bonyTA, tA;
	static Text3D timeTextGeom,concTextGeom;
	static boolean bugInScene = false;
	static boolean coordSysInScene = false;
	static boolean appearanceChanged = false;
//	 for coord sys drawing
	static Color3f xLineColor = new Color3f(1.0f,0.0f,0.0f);
	static Color3f yLineColor = new Color3f(0.0f,1.0f,0.0f);
	static Color3f zLineColor = new Color3f(0.0f,0.0f,1.0f);
	static Appearance xLineApp = new Appearance();
	static Appearance yLineApp = new Appearance();
	static Appearance zLineApp = new Appearance();
	static ColoringAttributes xLineCA = new ColoringAttributes(xLineColor, ColoringAttributes.FASTEST);
	static ColoringAttributes yLineCA = new ColoringAttributes(yLineColor, ColoringAttributes.FASTEST);
	static ColoringAttributes zLineCA = new ColoringAttributes(zLineColor, ColoringAttributes.FASTEST);
	double coordLineLength = 20*Env.parMMonoSize;
	LineArray xLine, yLine, zLine;
	Shape3D xLineShape, yLineShape, zLineShape;
	Pt3D xLineEndPt = new Pt3D();
	Pt3D yLineEndPt = new Pt3D();
	Pt3D zLineEndPt = new Pt3D();
	
	// Pt3Ds reused in collision calculations
	Pt3D forceVec = new Pt3D();
	Pt3D hemDist = new Pt3D();
	Pt3D objVec = new Pt3D();
	Pt3D yzVec = new Pt3D();
	
	public Bug (Pt3D coord, double length, double radius) {
		super (coord);
		this.length = length;
		this.radius = radius;
		cylLength = length-2*radius;
		halfCylLength = cylLength/2;
		cylVolume = (length-2*radius)*Math.pow(radius,2)*Math.PI;
		sphVolume = 4*Math.PI*Math.pow(radius,3)/3;
		bugVolume = cylVolume + sphVolume;  // volume in cubic microns
		double cubicMicronsPerLiter = 1e15;
		double bugVolumeInLiters = bugVolume/cubicMicronsPerLiter;
		double microMolesPerMole = 1e6;
		double milliMolesPerMole = 1e3;
		microMolarChangePerMonomer = microMolesPerMole/(bugVolumeInLiters*Env.AvogadroNum);		
		milliMolarChangePerMonomer = milliMolesPerMole/(bugVolumeInLiters*Env.AvogadroNum);		
		//talkln ("Bug contains " + String.valueOf(getParMConc()/microMolarChangePerMonomer));]
		//talkln ("microMolarChangePerMonomer = " + microMolarChangePerMonomer);
		talkln ("Bug volume in liters is " + bugVolumeInLiters);
	}
	
	public double getLength() {
		return length;
	}
	
	public double getRadius() {
		return radius;
	}
	
	public double getParMConc () {
		return Env.parMConc.getValue();
	}
	
	public double getNonHydroParMConc () {
		return Env.parMConcNonHydro.getValue();
	}
	
	public void takeParM (int numParM) {
		Env.parMConc.addToValue(-numParM*microMolarChangePerMonomer);
	}
	
	public void takeNonHydroParM (int numParM) {
		Env.parMConcNonHydro.addToValue(-numParM*microMolarChangePerMonomer);
	}
	
	public void putParM (int numParM) {
		Env.parMConc.addToValue(numParM*microMolarChangePerMonomer);
	}
	
	public void putNonHydroParM (int numParM) {
		Env.parMConcNonHydro.addToValue(numParM*microMolarChangePerMonomer);
	}
	
	public static int getMonsInSim () { 
		// calculate the assumed total number of monomers in the simulation
		int numInSol = (int)((Env.parMConc.getValue() + Env.parMConcNonHydro.getValue())/microMolarChangePerMonomer);
		return numInSol + FilSegment.getTotalMonCt();
	}
	
	public static void makeInitialBug () {
		Thing.theBug = new Bug(new Pt3D(0,0,0), Env.bugLength.getValue(), Env.bugRadius.getValue());
	}
	
	public Pt3D rdmPtInsideBug () {
		Pt3D pt = new Pt3D();
		double fracVolInCyl = cylVolume/bugVolume;
		if (Env.mtRNG.raw() < fracVolInCyl) {  //random point in cylinder
			double yC = Env.mtRNG.raw()*2-1;
			double zC = Env.mtRNG.raw()*2-1;
			while ((yC*yC + zC+zC > 1)) { zC = Env.mtRNG.raw()*2-1; }
			pt.y = yC*radius;
			pt.z = zC*radius;
			pt.x = (Env.mtRNG.raw()*2-1)*halfCylLength;
		} else {							// random point in one of the hemispheres
			pt = Pt3D.RandomUnitVec();
			pt.scale(radius*Env.mtRNG.raw());
			if (pt.x > 0) {
				pt.x += halfCylLength;
			} else {
				pt.x += -1*halfCylLength;
			}
		}
		pt.inc(coord);
		return pt;
	}
	
	public Pt3D rdmPtInsideBug (double objRadius) {
		Pt3D pt = new Pt3D();
		double fracVolInCyl = cylVolume/bugVolume;
		if (Env.mtRNG.raw() < fracVolInCyl) {  //random point in cylinder
			double yC = Env.mtRNG.raw()*2-1;
			double zC = Env.mtRNG.raw()*2-1;
			while ((yC*yC + zC*zC > 1)) { zC = Env.mtRNG.raw()*2-1; }
			pt.y = yC*(radius-objRadius);
			pt.z = zC*(radius-objRadius);

			pt.x = (Env.mtRNG.raw()*2-1)*halfCylLength;
		} else {							// random point in one of the hemispheres
			pt = Pt3D.RandomUnitVec();
			pt.scale((radius-objRadius)*Env.mtRNG.raw());
			if (pt.x > 0) {
				pt.x += halfCylLength;
			} else {
				pt.x += -1*halfCylLength;
			}
		}
		pt.inc(coord);
		return pt;
	}
	
	public CollisionEvent amIcolliding (Pt3D ctr, double R) {
		CollisionEvent CE = null;
		double dist = Pt3D.ptDist(ctr, coord);
		objVec.sub(ctr, coord);
		yzVec.copy(0, objVec.y, objVec.z);
		double yzDist = Pt3D.vecMag(yzVec);
		if (Math.abs(objVec.x) < halfCylLength) {	// check collisions in cylindrical section of bug
			if (yzDist+R < radius) {
				return CE;
			} else {
				double impingedist = Math.abs(yzDist + R - radius);
				yzVec.unitVec();
				yzVec.reverse();  // yzVec is useful as a force vector
				CE = new CollisionEvent(impingedist, yzVec);
				return CE;
			 }
			
		} else {		// check collisions in hemispherical caps
			if (objVec.x > 0) {	// positive x hemisphere
				hemDist.copy(coord.x+halfCylLength, coord.y, coord.z);
				double topdist = Pt3D.ptDist (ctr, hemDist);
				if (topdist + R < radius) {
					return CE;
				} else {
					double impingedist = Math.abs(topdist + R - radius);
					forceVec.unitVec(topdist, hemDist, ctr);
					CE = new CollisionEvent(impingedist, forceVec);
					return CE;
				}
			}
			if (objVec.x < 0) {  // negative x hemisphere
				hemDist.copy(coord.x-halfCylLength, coord.y, coord.z);
				double botdist = Pt3D.ptDist (ctr, hemDist);
				if (botdist + R < radius) {
					return CE;
				} else {
					double impingedist = Math.abs(botdist + R - radius);
					forceVec.unitVec(botdist, hemDist, ctr);
					CE = new CollisionEvent(impingedist, forceVec);
					return CE;
				}
			}
		
		}
		return CE;
	}
	
	public static void appearanceChange() {
		appearanceChanged = true;
		if (Env.paused) { theBug.updateGraphics();}
	}
	
	public void setWireFrameAppearance () {
		a.setLineAttributes(new LineAttributes(0.5f, LineAttributes.PATTERN_DASH,true));
		aSphCaps.setLineAttributes(new LineAttributes(0.5f, LineAttributes.PATTERN_DASH,true));;
		PolygonAttributes wireAtts = new PolygonAttributes(PolygonAttributes.POLYGON_LINE,PolygonAttributes.CULL_FRONT,0.0f,true);
		a.setPolygonAttributes(wireAtts);
		PolygonAttributes wireSphCapAtts = new PolygonAttributes(PolygonAttributes.POLYGON_LINE,PolygonAttributes.CULL_BACK,0.0f,true);
		aSphCaps.setPolygonAttributes(wireSphCapAtts);
	}
	
	public void setFillAppearance () {
		a.setLineAttributes(new LineAttributes(1.0f, LineAttributes.PATTERN_SOLID,true));
		aSphCaps.setLineAttributes(new LineAttributes(1.0f, LineAttributes.PATTERN_SOLID,true));
		PolygonAttributes filAtts = new PolygonAttributes(PolygonAttributes.POLYGON_FILL,PolygonAttributes.CULL_FRONT,0.0f);
		filAtts.setBackFaceNormalFlip(true);
		a.setPolygonAttributes(filAtts);
		PolygonAttributes filSphCapAtts = new PolygonAttributes(PolygonAttributes.POLYGON_FILL,PolygonAttributes.CULL_BACK,0.0f);
		filSphCapAtts.setBackFaceNormalFlip(true);
		aSphCaps.setPolygonAttributes(filSphCapAtts);
	}
	
	public void setCoarseWireAppearance () {
		if (Env.bugWiredCoarse) {
			a.setTransparencyAttributes(noSeeMeTA);
			aSphCaps.setTransparencyAttributes(noSeeMeTA);
			bonyApp.setTransparencyAttributes(bonyTA);
		} else {
			a.setTransparencyAttributes(tA);
			aSphCaps.setTransparencyAttributes(tA);
			bonyApp.setTransparencyAttributes(noSeeMeTA);
		}
	}
			
	
	public void updateTextInfoState() {
		infoG.detach();
		if (Env.infoIn3D) {
			g3d.addChild(infoG);
		}
	}
	
	public void makeGraphics () {
		setGraphicsCapabilities();	// set capabilities for BranchGroup, TransformGroup, and Appearance
		// no see um TransparencyAttributes
		noSeeMeTA = new TransparencyAttributes(TransparencyAttributes.NICEST,1.0f);
		
		// the main appearance
		m = new Material (ambientC,emissiveC,diffuseC,specularC,shiny);
		m.setCapability(Material.ALLOW_COMPONENT_WRITE);
		ColoringAttributes cA = new ColoringAttributes(1.0f,1.0f,1.0f,ColoringAttributes.NICEST);
		tA = new TransparencyAttributes (TransparencyAttributes.NICEST,Bug.initialTrans );
		tA.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
		a.setMaterial(m);
		a.setColoringAttributes(cA);
		a.setTransparencyAttributes(tA);
		
		// make alternate transparent appearance for cylinder end caps
		Appearance capA = new Appearance();
		ColoringAttributes capCA = new ColoringAttributes(0.0f,0.0f,0.0f,ColoringAttributes.NICEST);
		capA.setColoringAttributes(capCA);
		capA.setTransparencyAttributes(noSeeMeTA);
		capA.setLineAttributes(new LineAttributes(0.5f, LineAttributes.PATTERN_DASH_DOT,true));
		capA.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_POINT,PolygonAttributes.CULL_BACK,0.0f));
		
		// make another appearance for spherical ends
		aSphCaps = new Appearance ();
		aSphCaps.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
		aSphCaps.setCapability(Appearance.ALLOW_LINE_ATTRIBUTES_WRITE);
		aSphCaps.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		aSphCaps.setMaterial(m);
		aSphCaps.setColoringAttributes(cA);
		aSphCaps.setTransparencyAttributes(tA);
		
		// make another appearance for second cylinder
		bonyApp = new Appearance ();
		bonyApp.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
		bonyApp.setCapability(Appearance.ALLOW_LINE_ATTRIBUTES_WRITE);
		bonyApp.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		bonyApp.setMaterial(m);
		bonyApp.setColoringAttributes(cA);
		bonyTA = new TransparencyAttributes(TransparencyAttributes.NICEST,0.8f);
		bonyApp.setTransparencyAttributes(bonyTA);
		bonyApp.setLineAttributes(new LineAttributes(1.0f, LineAttributes.PATTERN_SOLID,true));
		PolygonAttributes bonyAtts = new PolygonAttributes(PolygonAttributes.POLYGON_LINE,PolygonAttributes.CULL_NONE,0.0f);
		bonyAtts.setBackFaceNormalFlip(true);
		bonyApp.setPolygonAttributes(bonyAtts);
		
		setCoarseWireAppearance();
		if (Env.bugWired){
			setWireFrameAppearance ();  // start out with the wireframe appearance
		} else {
			setFillAppearance();
		}
		
		//build the bug shape... first the cylindrical section.. 2 cylinders
		myCyl1 = new Cylinder((float)radius,(float)(cylLength),Primitive.GENERATE_NORMALS,80,80,a);
		myCyl1.setCapability(myCyl1.ENABLE_APPEARANCE_MODIFY);
		myCyl1.setAppearance(myCyl1.TOP,capA);
		myCyl1.setAppearance(myCyl1.BOTTOM,capA);
		Transform3D cylT3D = new Transform3D();
		cylT3D.rotZ(Math.PI/2);
		TransformGroup cylTG = new TransformGroup(cylT3D);
		cylTG.addChild(myCyl1);
		
		myCyl2 = new Cylinder((float)(radius+cellThickness),(float)cylLength,Primitive.GENERATE_NORMALS,16,16,bonyApp);
		myCyl2.setCapability(myCyl2.ENABLE_APPEARANCE_MODIFY);
		myCyl2.setAppearance(myCyl1.TOP,capA);
		myCyl2.setAppearance(myCyl1.BOTTOM,capA);
		Transform3D cyl2T3D = new Transform3D();
		cyl2T3D.rotZ(Math.PI/2);
		TransformGroup cyl2TG = new TransformGroup(cyl2T3D);
		cyl2TG.addChild(myCyl2);
		
		// right hemispherical cap (arch)
		rArch = new SphereSection (0.0, Math.PI/2, 60, 2*Math.PI, 0.0, 80, radius, aSphCaps);
		rArch.setCapability(rArch.ALLOW_APPEARANCE_WRITE);
		Transform3D rArchT3D = new Transform3D();
		rArchT3D.rotZ(-Math.PI/2);
		Transform3D tweakRot = new Transform3D();
		tweakRot.rotY(6*Math.PI/180);
		//rArchT3D.mul(tweakRot);
		rArchT3D.setTranslation(new Vector3d(cylLength/2,0,0));
		TransformGroup rArchTG = new TransformGroup(rArchT3D);
		
		// bony right hemispherical cap (arch)
		rArchBony = new SphereSection (0.0, Math.PI/2, 12, 2*Math.PI, 0.0, 17, radius, bonyApp);
		rArchBony.setCapability(rArch.ALLOW_APPEARANCE_WRITE);
		rArchTG.addChild(rArch);
		rArchTG.addChild(rArchBony);

		// left hemispherical cap (arch)
		lArch = new SphereSection (0.0, Math.PI/2, 60, 2*Math.PI, 0.0, 80, radius, aSphCaps);
		lArch.setCapability(lArch.ALLOW_APPEARANCE_WRITE);
		Transform3D lArchT3D = new Transform3D();
		lArchT3D.rotZ(Math.PI/2);
		lArchT3D.setTranslation(new Vector3d(-cylLength/2,0,0));
		TransformGroup lArchTG = new TransformGroup(lArchT3D);
		
		// left hemispherical cap (arch)
		lArchBony = new SphereSection (0.0, Math.PI/2, 12, 2*Math.PI, 0.0, 17, radius, bonyApp);
		lArchBony.setCapability(lArch.ALLOW_APPEARANCE_WRITE);
		lArchTG.addChild(lArch);
		lArchTG.addChild(lArchBony);
		
		Appearance textAppear = new Appearance();
		Color3f noColor = new Color3f (0f,0f,0f);
		Color3f textEmitColor = new Color3f(0.7f,0.7f,0.7f);
		Color3f textDiffColor = new Color3f(0.7f,0.7f,0.7f);
		Color3f textSpecColor = new Color3f(1.0f,1.0f,1.0f);
	    textAppear.setMaterial(new Material(noColor,textEmitColor,textDiffColor,textSpecColor,128));
	    Font textFont = new Font("Helvetica",Font.ITALIC,2);
	    Font3D font3D = new Font3D(textFont,0.001,new FontExtrusion());
		
	    timeTextGeom = new Text3D(font3D, new String("Time: " + timeFormat.format(0)));
	    timeTextGeom.setCapability(Text3D.ALLOW_STRING_WRITE);
	    timeTextGeom.setAlignment(Text3D.ALIGN_FIRST);
	    Shape3D timeTextShape = new Shape3D();
	    timeTextShape.setGeometry(timeTextGeom);
	    timeTextShape.setAppearance(textAppear);
	    Transform3D timeTextT3D = new Transform3D();
	    TransformGroup timeTextTG = new TransformGroup();
	    timeTextT3D.setScale(0.03);
	    timeTextT3D.setTranslation(new Vector3d(-length/2+radius,radius+0.03,0));
	    timeTextTG.setTransform(timeTextT3D);
	    timeTextTG.addChild(timeTextShape);
	    
	    concTextGeom = new Text3D(font3D, new String("ParM: " + concFormat.format(Env.parMConc.getValue())));
	    concTextGeom.setCapability(Text3D.ALLOW_STRING_WRITE);
	    concTextGeom.setAlignment(Text3D.ALIGN_FIRST);
	    Shape3D concTextShape = new Shape3D();
	    concTextShape.setGeometry(concTextGeom);
	    concTextShape.setAppearance(textAppear);
	    Transform3D concTextT3D = new Transform3D();
	    TransformGroup concTextTG = new TransformGroup();
	    concTextT3D.setScale(0.03);
	    concTextT3D.setTranslation(new Vector3d(-.45,radius+0.03,0));
	    concTextTG.setTransform(concTextT3D);
	    concTextTG.addChild(concTextShape);
	    
	    infoG = new BranchGroup();
	    infoG.setCapability(BranchGroup.ALLOW_DETACH);
	    infoG.addChild(timeTextTG);
	    //infoG.addChild(concTextTG);
		
		g3d.setTransform(t3d);
		
		g3d.addChild(cylTG);
		g3d.addChild(cyl2TG);
		g3d.addChild(lArchTG);
		g3d.addChild(rArchTG);
		
		if (Env.infoIn3D) { g3d.addChild(infoG);}
		
		// coordinate system
		xLineApp.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
		yLineApp.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
		zLineApp.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
		xLineApp.setColoringAttributes(xLineCA);
		yLineApp.setColoringAttributes(yLineCA);
		zLineApp.setColoringAttributes(zLineCA);
		// x-axis
		xLine = new LineArray(2,LineArray.COORDINATES);
		xLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		xLineEndPt.add(coord,coordLineLength,uVec);
		xLine.setCoordinate(0,coord);
		xLine.setCoordinate(1,xLineEndPt);
		xLineShape = new Shape3D();
		xLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		xLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		xLineShape.setGeometry(xLine);
		xLineShape.setAppearance(xLineApp);
		// y-axis
		yLine = new LineArray(2,LineArray.COORDINATES);
		yLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		yLineEndPt.add(coord,coordLineLength,yVec);
		yLine.setCoordinate(0,coord);
		yLine.setCoordinate(1,yLineEndPt);
		yLineShape = new Shape3D();
		yLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		yLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		yLineShape.setGeometry(yLine);
		yLineShape.setAppearance(yLineApp);
		// z-axis
		zLine = new LineArray(2,LineArray.COORDINATES);
		zLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		zLineEndPt.add(coord,coordLineLength,zVec);
		zLine.setCoordinate(0,coord);
		zLine.setCoordinate(1,zLineEndPt);
		zLineShape = new Shape3D();
		zLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		zLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		zLineShape.setGeometry(zLine);
		zLineShape.setAppearance(zLineApp);
		BranchGroup coordSysG = new BranchGroup();
		coordSysG.addChild(xLineShape);
		coordSysG.addChild(yLineShape);
		coordSysG.addChild(zLineShape);
	
		bugG = new BranchGroup();
		bugG.setCapability(BranchGroup.ALLOW_DETACH);
		bugG.addChild(g3d);

		if (!Env.biochemOnly.isActive() & !Env.bugOff) {	// two cases where we don't want bug in the mix
			G.addChild(bugG);
			bugInScene = true;
		}
		
		if (Env.bugCoordSysOn) {
			bugG.addChild(coordSysG);
		}
		
		G.compile();
		graphicsMade = true;
	}
	
	public void updateGraphics () {
		if (Env.bugOff | Env.biochemOnly.isActive()) { 
			if (bugInScene) {
				bugG.detach();
				bugInScene = false;
			}
		} else {
			if (!bugInScene) {
				G.addChild(bugG);
				bugInScene = true;
			}
		}
		
		if (Env.infoIn3D) {
			timeTextGeom.setString("Time: " + timeFormat.format(Env.simulationTime));
			//concTextGeom.setString("ParM: " + concFormat.format(Env.parMConc.getValue()));
		}
		
		
		if ((appearanceChanged)&(!Env.biochemOnly.isActive())) {
				setCoarseWireAppearance();
				if (Env.bugWired) { setWireFrameAppearance(); } else { setFillAppearance(); }
				appearanceChanged = false;
		}
	}
}
